<?php

namespace App\Service;

use App\Jobs\DoorDeviceJob;
use Illuminate\Support\Facades\Redis;

class InOutOptionService extends Service
{
    const OrderKeyQueueIn = 'InOrderNumber';
    const OrderKeyQueueOut = 'OutOrderNumber';
    const Key = 'FHCK';

    public function setOrderNumber($data){
        if(! empty($data['data']['tagList'])){
            $in = $out = [];$site = "";
            $device_id = $data['data']['id'] ?? 0; //设备id
            foreach ($data['data']['tagList'] as $value){
                if(empty($site)) $site = $value['remark'] ?? ""; //配置成站点
                if($value['direction'] == '1'){
                    //（判断到有发货出库的入库标识 存入缓存）
                    if(Redis::exists($device_id . self::Key)){
                        Redis::lpush($device_id . self::Key, json_encode($value));
                    }else{
                        //入库 直接生成入库单
                        if(! in_array($value['epc'],$in)) $in[] = $value['epc'];
                    }
                }elseif ($value['direction'] == '2'){
                    //（判断到有发货出库的出库标识 存入缓存）
                    if(Redis::exists($device_id)){
                        Redis::lpush($device_id . self::Key, json_encode($value));
                    }else{
                        //出库 直接生成出库单
                        if(! in_array($value['epc'],$out)) $out[] = $value['epc'];
                    }
                }
            }
            if(! empty($in)) dispatch(new DoorDeviceJob($in,1,$site,$device_id))->onQueue('door_device');
            if(! empty($out)) dispatch(new DoorDeviceJob($out,2,$site,$device_id))->onQueue('door_device');
        }
    }

    public function getOrderNumber($data,$site){
        //获取某个站点的数据
        $tmp = [];
        $result = [];

        //发货出库
        $key = $data['device_id'] . self::Key;
        if(Redis::exists(self::Key)) {
            //发货
            while ($item = Redis::lpop($key)) {
                $order = json_decode($item, true);
                $order_it = $order['epc'] . $order['direction'];
                if(! in_array($order_it, $tmp)){
                    $order['epc'] = hex2bin($order['epc']);
                    $order['site'] = $site;
                    $result[] = $order;
                    $tmp[] = $order_it;
                }
            }

            return [true, $result];
        }

        //入库
        $key = $data['device_id'] . self::OrderKeyQueueIn;
        $res = Redis::get($key);
        if(! empty($res)){
            $return  = json_decode($res,true);
            foreach ($return as $value){
                $result[] = [
                    'epc' => $value,
                    'direction' => '1',
                ];
            }
            return [true, $result];
        }

        //发货
        $key = $data['device_id'] . self::OrderKeyQueueOut;
        $res = Redis::get($key);
        if(! empty($res)){
            $return  = json_decode($res,true);
            foreach ($return as $value){
                $result[] = [
                    'epc' => $value,
                    'direction' => '2',
                ];
            }
            return [true, $result];
        }

        return [true, $result];
    }

    //设置为发货出库
    public function setFhMessage($data,$site){
        $key = $data['device_id'] . self::Key;
        if(empty($data['fh'])){
            if(Redis::exists(self::Key)) Redis::del(self::Key);
        }else{
            if (Redis::setnx($key, 1)) Redis::expire($key, 60); //多少秒后过期
        }
    }
}
